@@ -69,78 +69,89 @@ class SMBDRSUAPI:
 		return True,None
 		
 	
-	@red
 	async def connect(self, open = False):
-		epm = EPM(self.connection, protocol = 'ncacn_ip_tcp')
-		await rr(epm.connect())
-		stringBinding, _ = await rr(epm.map(drsuapi.MSRPC_UUID_DRSUAPI))
-		self.dce = epm.get_connection_from_stringbinding(stringBinding)
+		try:
+			epm = EPM(self.connection, protocol = 'ncacn_ip_tcp')
+			_, err = await epm.connect()
+			if err is not None:
+				raise err
+			stringBinding, _ = await rr(epm.map(drsuapi.MSRPC_UUID_DRSUAPI))
+			self.dce = epm.get_connection_from_stringbinding(stringBinding)
 
-		#the line below must be set!
-		self.dce.set_auth_level(RPC_C_AUTHN_LEVEL_PKT_PRIVACY)
+			#the line below must be set!
+			self.dce.set_auth_level(RPC_C_AUTHN_LEVEL_PKT_PRIVACY)
 
-		await rr(self.dce.connect())
+			_, err = await self.dce.connect()
+			if err is not None:
+				raise err
+
+			
+			if open == True:
+				_, err = await self.open()
+				if err is not None:
+					raise err
+			return True, None
+		except Exception as e:
+			return False, e
 		
-		if open == True:
-			await rr(self.open())
-		return True,None
-	
-	@red
 	async def open(self):
-		if not self.dce:
-			await rr(self.connect())
-		
-		await rr(self.dce.bind(drsuapi.MSRPC_UUID_DRSUAPI))
-		request = drsuapi.DRSBind()
-		request['puuidClientDsa'] = drsuapi.NTDSAPI_CLIENT_GUID
-		drs = drsuapi.DRS_EXTENSIONS_INT()
-		drs['cb'] = len(drs) #- 4
-		drs['dwFlags'] = drsuapi.DRS_EXT_GETCHGREQ_V6 | drsuapi.DRS_EXT_GETCHGREPLY_V6 | drsuapi.DRS_EXT_GETCHGREQ_V8 | \
-						 drsuapi.DRS_EXT_STRONG_ENCRYPTION
-		drs['SiteObjGuid'] = drsuapi.NULLGUID
-		drs['Pid'] = 0
-		drs['dwReplEpoch'] = 0
-		drs['dwFlagsExt'] = 0
-		drs['ConfigObjGUID'] = drsuapi.NULLGUID
-		# I'm uber potential (c) Ben
-		drs['dwExtCaps'] = 0xffffffff
-		request['pextClient']['cb'] = len(drs)
-		request['pextClient']['rgb'] = list(drs.getData())
-		resp,_ = await rr(self.dce.request(request))
-		
-		# Let's dig into the answer to check the dwReplEpoch. This field should match the one we send as part of
-		# DRSBind's DRS_EXTENSIONS_INT(). If not, it will fail later when trying to sync data.
-		drsExtensionsInt = drsuapi.DRS_EXTENSIONS_INT()
-
-		# If dwExtCaps is not included in the answer, let's just add it so we can unpack DRS_EXTENSIONS_INT right.
-		ppextServer = b''.join(resp['ppextServer']['rgb']) + b'\x00' * (
-		len(drsuapi.DRS_EXTENSIONS_INT()) - resp['ppextServer']['cb'])
-		drsExtensionsInt.fromString(ppextServer)
-
-		if drsExtensionsInt['dwReplEpoch'] != 0:
-			# Different epoch, we have to call DRSBind again
-			if logger.level == logging.DEBUG:
-				logger.debug("DC's dwReplEpoch != 0, setting it to %d and calling DRSBind again" % drsExtensionsInt[
-					'dwReplEpoch'])
-			drs['dwReplEpoch'] = drsExtensionsInt['dwReplEpoch']
+		try:
+			if not self.dce:
+				await rr(self.connect())
+			
+			await rr(self.dce.bind(drsuapi.MSRPC_UUID_DRSUAPI))
+			request = drsuapi.DRSBind()
+			request['puuidClientDsa'] = drsuapi.NTDSAPI_CLIENT_GUID
+			drs = drsuapi.DRS_EXTENSIONS_INT()
+			drs['cb'] = len(drs) #- 4
+			drs['dwFlags'] = drsuapi.DRS_EXT_GETCHGREQ_V6 | drsuapi.DRS_EXT_GETCHGREPLY_V6 | drsuapi.DRS_EXT_GETCHGREQ_V8 | \
+							drsuapi.DRS_EXT_STRONG_ENCRYPTION
+			drs['SiteObjGuid'] = drsuapi.NULLGUID
+			drs['Pid'] = 0
+			drs['dwReplEpoch'] = 0
+			drs['dwFlagsExt'] = 0
+			drs['ConfigObjGUID'] = drsuapi.NULLGUID
+			# I'm uber potential (c) Ben
+			drs['dwExtCaps'] = 0xffffffff
 			request['pextClient']['cb'] = len(drs)
 			request['pextClient']['rgb'] = list(drs.getData())
 			resp,_ = await rr(self.dce.request(request))
+			
+			# Let's dig into the answer to check the dwReplEpoch. This field should match the one we send as part of
+			# DRSBind's DRS_EXTENSIONS_INT(). If not, it will fail later when trying to sync data.
+			drsExtensionsInt = drsuapi.DRS_EXTENSIONS_INT()
 
-		self.handle = resp['phDrs']
+			# If dwExtCaps is not included in the answer, let's just add it so we can unpack DRS_EXTENSIONS_INT right.
+			ppextServer = b''.join(resp['ppextServer']['rgb']) + b'\x00' * (
+			len(drsuapi.DRS_EXTENSIONS_INT()) - resp['ppextServer']['cb'])
+			drsExtensionsInt.fromString(ppextServer)
 
-		# Now let's get the NtdsDsaObjectGuid UUID to use when querying NCChanges
-		resp, _ = await rr(drsuapi.hDRSDomainControllerInfo(self.dce, self.handle, self.domainname, 2))
-		if logger.level == logging.DEBUG:
-			logger.debug('DRSDomainControllerInfo() answer %s' % resp.dump())
+			if drsExtensionsInt['dwReplEpoch'] != 0:
+				# Different epoch, we have to call DRSBind again
+				if logger.level == logging.DEBUG:
+					logger.debug("DC's dwReplEpoch != 0, setting it to %d and calling DRSBind again" % drsExtensionsInt[
+						'dwReplEpoch'])
+				drs['dwReplEpoch'] = drsExtensionsInt['dwReplEpoch']
+				request['pextClient']['cb'] = len(drs)
+				request['pextClient']['rgb'] = list(drs.getData())
+				resp,_ = await rr(self.dce.request(request))
 
-		if resp['pmsgOut']['V2']['cItems'] > 0:
-			self.__NtdsDsaObjectGuid = resp['pmsgOut']['V2']['rItems'][0]['NtdsDsaObjectGuid']
-		else:
-			logger.debug("Couldn't get DC info for domain %s" % self.domainname)
-			raise Exception('Fatal, aborting!')
+			self.handle = resp['phDrs']
 
-		return True,None
+			# Now let's get the NtdsDsaObjectGuid UUID to use when querying NCChanges
+			resp, _ = await rr(drsuapi.hDRSDomainControllerInfo(self.dce, self.handle, self.domainname, 2))
+			if logger.level == logging.DEBUG:
+				logger.debug('DRSDomainControllerInfo() answer %s' % resp.dump())
+
+			if resp['pmsgOut']['V2']['cItems'] > 0:
+				self.__NtdsDsaObjectGuid = resp['pmsgOut']['V2']['rItems'][0]['NtdsDsaObjectGuid']
+			else:
+				logger.debug("Couldn't get DC info for domain %s" % self.domainname)
+				raise Exception('Fatal, aborting!')
+
+			return True,None
+		except Exception as e:
+			return None, e
 	
 	async def get_user_secrets(self, username):
 		try:
@@ -397,6 +408,7 @@ class SMBDRSUAPI:
 			data, err = await self.dce.request(request)
 			return data, err
 		except Exception as e:
+			print('err!')
 			return None, e
 
 
