@@ -30,8 +30,11 @@ class FileTargetGen:
 			with open(self.filename, 'r') as f:
 				for line in f:
 					line = line.strip()
+					if line == '':
+						continue
 					await target_q.put((str(uuid.uuid4()), line))
 					await asyncio.sleep(0)
+					cnt += 1
 			return cnt, None
 		except Exception as e:
 			return cnt, e
@@ -69,7 +72,6 @@ class SMBFileEnum:
 
 	async def __executor(self, tid, target):
 		try:
-			
 			connection = self.smb_mgr.create_connection_newtarget(target)
 			async with connection:
 				_, err = await connection.login()
@@ -77,10 +79,10 @@ class SMBFileEnum:
 					raise err
 
 				machine = SMBMachine(connection)
-
 				async for path, otype, err in machine.enum_all_recursively(depth = self.depth):
 					er = EnumResult(tid, target, (path, otype, err))
 					await self.res_q.put(er)
+
 		except asyncio.CancelledError:
 			return
 		except Exception as e:
@@ -111,20 +113,26 @@ class SMBFileEnum:
 	async def result_processing(self):
 		try:
 			while True:
-				er = await self.res_q.get()
-				if er.status == EnumResultStatus.FINISHED:
-					self.__total_finished += 1
-					if self.__total_finished == self.__total_targets and self.__gens_finished is True:
-						asyncio.create_task(self.terminate())
-						return
-				
-				if er.result is not None:
-					path, otype, err = er.result
-					if otype is not None:
-						print('[%s] %s' % (otype[0].upper(), path))
-					if err is not None:
-						print('[E] %s %s' % (err, path))
-				
+				try:
+					er = await self.res_q.get()
+					if er.status == EnumResultStatus.FINISHED:
+						self.__total_finished += 1
+						print('[P][%s/%s][%s]' % (self.__total_targets, self.__total_finished, str(self.__gens_finished)))
+						if self.__total_finished == self.__total_targets and self.__gens_finished is True:
+							asyncio.create_task(self.terminate())
+							return
+					
+					if er.result is not None:
+						path, otype, err = er.result
+						if otype is not None:
+							print('[%s] %s' % (otype[0].upper(), path))
+						if err is not None:
+							print('[E] %s %s' % (err, path))
+				except asyncio.CancelledError:
+					return
+				except Exception as e:
+					print(e)
+					continue
 		except asyncio.CancelledError:
 			return
 		except Exception as e:
@@ -181,21 +189,30 @@ async def amain():
 	parser = argparse.ArgumentParser(description='SMB Share enumerator')
 	parser.add_argument('-d', '--depth', type=int, default=3, help='Recursion depth, -1 means infinite')
 	parser.add_argument('-w', '--smb-worker-count', type=int, default=100, help='Parallell count')
-	parser.add_argument('-t', '--targets-file', help='Targets file, one ip or hostname per line')
-	parser.add_argument('-m', '--machine', help='Targetmachoine ip or hostname, can be stacked')
-	parser.add_argument('smb_url', help = 'Connection string that describes the authentication and target. Example: smb+ntlm-password://TEST\\Administrator:password@10.10.10.2')
+	parser.add_argument('smb_url', help='Connection URL base, target can be anything. Example: "smb2+ntlm-password://TEST\\victim@test"')
+	parser.add_argument('targets', nargs='+', help = 'Hostname or IP address or file with a list of targets')
 	args = parser.parse_args()
 
 	logger.setLevel(100)
 	enumerator = SMBFileEnum(args.smb_url, worker_count = args.smb_worker_count, depth = args.depth)
 	
-	if args.targets_file is not None:
-		enumerator.target_gens.append(FileTargetGen(args.targets_file))
-	if args.machine is not None:
-		enumerator.target_gens.append(ListTargetGen(args.machine))
+	notfile = []
+	for target in args.targets:
+		try:
+			f = open(target, 'r')
+			f.close()
+			enumerator.target_gens.append(FileTargetGen(target))
+		except:
+			notfile.append(target)
+	
+	if len(notfile) > 0:
+		enumerator.target_gens.append(ListTargetGen(notfile))
 
+	if len(enumerator.target_gens) == 0:
+		print('[-] No suitable targets were found!')
+		return
+		
 	await enumerator.run()
-	print('Done!')
 
 def main():
 	asyncio.run(amain())
